using Demo.Domain.Models;
using Demo.Data.Repository;

namespace Demo.Domain.UseCase
{
    public class UserUseCase : IUserUseCase
    {
		private readonly IUserRepository _repositoryUserImpl;
        private readonly IGroupRepository _repositoryGroupImpl;

        public UserUseCase(IGroupRepository repositoryGroupImpl, IUserRepository repositoryUserImpl)
        {
            _repositoryGroupImpl = repositoryGroupImpl;
            _repositoryUserImpl = repositoryUserImpl;
        }

        public List<Group> GetAllGroups() => _repositoryGroupImpl.GetAllGroup()
        .Select(it => new Group{ID = it.ID, Name = it.Name}).ToList();

        public List<User> GetAllUsers() => _repositoryUserImpl.GetAllUser()
        .Join(_repositoryGroupImpl.GetAllGroup(),
        user => user.GroupID,
        group => group.ID,
        (user, group) => new User{
            FIO = user.FIO,
            Guid = user.Guid, 
            Group = new Group{
                ID = group.ID, 
                Name = group.Name}
            }
        ).ToList();

        public User GetUserByGuid(Guid userGuid){
            UserLocalEntity? userLocalEntity = _repositoryUserImpl.GetUserById(userGuid);
            if (userLocalEntity == null) throw new Exception("bello");
            Group? group = GetAllGroups().FirstOrDefault(it => userLocalEntity.GroupID == it.ID);
            if (group == null) throw new Exception("bello");
            return new User{
                FIO = userLocalEntity.FIO,
                Guid = userLocalEntity.Guid,
                Group = new Group{
                    ID = group.ID,
                    Name = group.Name}
            };
        }

        public bool RemoveUserByGuid(Guid userGuid) {
           return _repositoryUserImpl.RemoveUserByGuid(userGuid);
        }

        public User UpdateUser(User user)
        {
            UserLocalEntity userLocalEntity = new UserLocalEntity
            {
                FIO = user.FIO,
                Guid = user.Guid,
                GroupID = user.Group.ID
            };
            UserLocalEntity? result = _repositoryUserImpl.UpdateUserById(userLocalEntity);
            if (result == null)
            {
                throw new Exception("Не удалось обновить пользователя: пользователь не найден.");
            }
            Group? group = GetAllGroups().FirstOrDefault(it => result.GroupID == it.ID);
            if (group == null)
            {
                throw new Exception("Не удалось обновить пользователя: группа не найдена.");
            }
            return new User
            {
                FIO = result.FIO,
                Guid = result.Guid,
                Group = group
            };
        }
    }
}
